import { catchError, map } from 'rxjs/operators';
import { CompanyActionTypes } from './company.action.type';
import {
  CompanyActionUnion,
  CreateFormAddCompanyActionSuccess,
  CreateFormAddCompanyActionError,
  SaveNewCompanyActionSuccess,
  SaveNewCompanyActionError,
} from './company.actions';
import { Observable, of, mergeMap } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { CompanyService } from './../../shared/services/company.service';
import { Injectable } from '@angular/core';
import { Company } from 'src/app/shared/models/company/company.model';

@Injectable()
export class CompanyCreateEffects {
  constructor(
    private companyService: CompanyService,
    private companyActions: Actions
  ) {}

  //create form to add company
  createFormAddCompanyEffect: Observable<CompanyActionUnion> = createEffect(
    () =>
      this.companyActions.pipe(
        ofType(CompanyActionTypes.CREATE_FORM_ADD_COMPANY),
        map(() => new CreateFormAddCompanyActionSuccess({})),
        catchError((err) => of(new CreateFormAddCompanyActionError(err)))
      )
  );

  // save new company
  saveNewCompany: Observable<CompanyActionUnion> = createEffect(() =>
    this.companyActions.pipe(
      ofType(CompanyActionTypes.SAVE_NEW_CAMPANY),
      mergeMap((action: CompanyActionUnion) => {
        return this.companyService.createCompanyService(action.payload).pipe(
          map((company: Company) => new SaveNewCompanyActionSuccess(company)),
          catchError((err) => of(new SaveNewCompanyActionError(err)))
        );
      })
    )
  );
}
