import { catchError, map } from 'rxjs/operators';
import { ProjectActionTypes } from './project.action.type';
import {
  ProjectActionUnion,
  CreateFormAddProjectActionSuccess,
  CreateFormAddProjectActionError,
  SaveNewProjectActionSuccess,
  SaveNewProjectActionError,
} from './project.action';
import { Observable, of, mergeMap } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { ProjectService } from './../../shared/services/project.service';
import { Injectable } from '@angular/core';
import { Project } from 'src/app/shared/models/project/project.model';

@Injectable()
export class ProjectCreateEffects {
  constructor(
    private projectService: ProjectService,
    private projectActions: Actions
  ) {}

  //create form to add new project
  createFormAddProjectEffect: Observable<ProjectActionUnion> = createEffect(
    () =>
      this.projectActions.pipe(
        ofType(ProjectActionTypes.CREATE_FORM_ADD_PROJECT),
        map(() => new CreateFormAddProjectActionSuccess({}, {}, {})),
        catchError((err) =>
          of(new CreateFormAddProjectActionError(err, {}, {}))
        )
      )
  );

  //save new project
  saveNewProjectEffect: Observable<ProjectActionUnion> = createEffect(() =>
    this.projectActions.pipe(
      ofType(ProjectActionTypes.SAVE_NEW_PROJECT),
      mergeMap((action: ProjectActionUnion) => {
        return this.projectService
          .createProjectService(action.payload)
          .pipe(
            map(
              (project: Project) =>
                new SaveNewProjectActionSuccess(
                  project
                )
            ),
            catchError((err) =>
              of(
                new SaveNewProjectActionError(
                  err
                )
              )
            )
          );
      })
    )
  );
}
