import { catchError, map } from 'rxjs/operators';
import { ProjectActionTypes } from './project.action.type';
import {
  ProjectActionUnion,
  CreateFormUpdatedProjectActionSuccess,
  CreateFormUpdatedProjectActionError,
  UpdateProjectActionSuccess,
  UpdateProjectActionError,
} from './project.action';
import { Observable, mergeMap, of } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { ProjectService } from './../../shared/services/project.service';
import { Injectable } from '@angular/core';
import { Project } from 'src/app/shared/models/project/project.model';

@Injectable()
export class ProjectUpdateEffect {
  constructor(
    private projectService: ProjectService,
    private projectActions: Actions
  ) {}

  //create form to update project
  createFormUpdateProject: Observable<ProjectActionUnion> = createEffect(() =>
    this.projectActions.pipe(
      ofType(ProjectActionTypes.CREATE_FORM_UPDATE_PROJECT),
      mergeMap((action: ProjectActionUnion) => {
        return this.projectService
          .getProjectByIDService(action.payload.projectID)
          .pipe(
            map(
              (project: Project) =>
                new CreateFormUpdatedProjectActionSuccess(project, {}, {})
            ),
            catchError((err) =>
              of(new CreateFormUpdatedProjectActionError(err, {}, {}))
            )
          );
      })
    )
  );

  // update project
  updateProjectEffect: Observable<ProjectActionUnion> = createEffect(() =>
    this.projectActions.pipe(
      ofType(ProjectActionTypes.UPDATE_PROJECT),
      mergeMap((action: ProjectActionUnion) => {
        return this.projectService
          .updateProjectService(action.payload)
          .pipe(
            map((project: Project) =>new UpdateProjectActionSuccess(project)
            ),
            catchError(err=> of (new UpdateProjectActionError(err)))
          );
      })
    )
  );

}
