import { AddressActionTypes } from './address.action.type';
import { Action } from '@ngrx/store';
import { MsgState } from '../msg.state';
import { AddressState } from './address.state';
import { AddressActionUnion } from './address.action';
import { Address } from 'src/app/shared/models/user/address.model';

const addressInitialState: AddressState = {
  currentAddress: null,
  msgState: MsgState.INIT_STATE,
  addressesList: [],
  errorMsg: 'no error',
};

export function addressReducer(
  state: AddressState = addressInitialState,
  action: Action
): AddressState {
  switch (action.type) {

    //get all addresses
    case AddressActionTypes.GET_ALL_ADDRESSES:
      return {
        ...state,
        msgState: MsgState.LOADING,
      };
    case AddressActionTypes.GET_ALL_ADDRESSES_SUCCESS:
      return {
        ...state,
        msgState: MsgState.LOADED,
        addressesList: (<AddressActionUnion>action).payload,
      };
    case AddressActionTypes.GET_ALL_ADDRESSES_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg: (<AddressActionUnion>action).payload,
      };

    // create form to add new address
    case AddressActionTypes.CREATE_FORM_ADD_ADDRESS:
      return {
        ...state,
        msgState: MsgState.FORM_LOADING,
      };
    case AddressActionTypes.CREATE_FORM_ADD_ADDRESS_SUCCESS:
      return {
        ...state,
        msgState: MsgState.FORM_LOADED,
      };
    case AddressActionTypes.CREATE_FORM_ADD_ADDRESS_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg: (<AddressActionUnion>action).payload,
      };

    // save new address
    case AddressActionTypes.SAVE_NEW_ADDRESS:
      return {
        ...state,
        msgState: MsgState.ADD,
      };
    case AddressActionTypes.SAVE_NEW_ADDRESS_SUCCESS:
      let newAddress: Address = (<AddressActionUnion>action).payload;
      let addressesListCopy1 = [...state.addressesList];
      addressesListCopy1.push(newAddress);
      return {
        ...state,
        msgState: MsgState.ADDED,
      };
    case AddressActionTypes.SAVE_NEW_ADDRESS_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg: (<AddressActionUnion>action).payload,
      };

    // create form to update address
    case AddressActionTypes.CREATE_FORM_UPDATE_ADDRESS:
      return {
        ...state,
        msgState: MsgState.FORM_LOADING,
      };
    case AddressActionTypes.CREATE_FORM_UPDATE_ADDRESS_SUCCESS:
      let addressToUpdate: Address = (<AddressActionUnion> action).payload;
      return {
        ...state,
        msgState: MsgState.FORM_LOADED,
        currentAddress:addressToUpdate
      };
    case AddressActionTypes.CREATE_FORM_UPDATE_ADDRESS_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg: (<AddressActionUnion>action).payload,
      };

    //update address
    case AddressActionTypes.UPDATE_ADDRESS:
      return {
        ...state,
        msgState: MsgState.UPDATE,
      };
    case AddressActionTypes.UPDATE_ADDRESS_SUCCESS:
      let updatedAddress: Address = (<AddressActionUnion>action).payload;
      let addressesListCopy2: Array<Address> = [...state.addressesList];
      addressesListCopy2.map((address) =>
        address.addressID == updatedAddress.addressID ? updatedAddress : address
      );
      return{
        ...state,
        msgState: MsgState.UPDATED,
        addressesList: addressesListCopy2
      };
    case AddressActionTypes.UPDATE_ADDRESS_ERROR:
      return{
        ...state,
        msgState:MsgState.ERR,
        errorMsg: (<AddressActionUnion>action).payload,
      };

    // delete address
    case AddressActionTypes.DELETE_ADDRESS:
      return{
        ...state,
        msgState:MsgState.DELETE,
        currentAddress: (<AddressActionUnion>action).payload
      };
    case AddressActionTypes.DELETE_ADDRESS_SUCCESS:
      let deletedAddress : Address = (<AddressActionUnion> action).payload;
      let deletedAddressIndex = state.addressesList.indexOf(deletedAddress);
      let addressesListCopy3 : Array<Address> = [...state.addressesList];
      addressesListCopy3.splice(deletedAddressIndex,1);

      return{
        ...state,
        msgState:MsgState.DELETED,
        addressesList: addressesListCopy3,
        currentAddress: deletedAddress
      };

    default:
      return { ...state };
  }
}
