import { UserService } from './../../shared/services/user.service';
import { catchError } from 'rxjs/operators';
import { UserActionTypes } from './user.action.type';
import {
  UserActionUnion,
  CreateFormAddUserActionSuccess,
  CreateFormAddUserActionError,
  SaveNewUserActionSuccess,
  SaveNewUserActionError,
} from './user.actions';
import { map, Observable, of, mergeMap } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Injectable } from '@angular/core';
import { User } from 'src/app/shared/models/user/user.model';

@Injectable()
export class UserCreateEffects {
  constructor(private actions: Actions, private userService: UserService) {}

  createUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
    this.actions.pipe(
      ofType(UserActionTypes.CREATE_FORM_ADD_USER),
      map(
        (action: UserActionUnion) => new CreateFormAddUserActionSuccess({}, {})
      ),
      catchError((err) => of(new CreateFormAddUserActionError(err, {})))
    )
  );

  saveNewUserEffect: Observable<UserActionUnion> = createEffect(() =>
    this.actions.pipe(
      ofType(UserActionTypes.SAVE_NEW_USER),
      mergeMap((action: UserActionUnion) => {
        return this.userService
          .createUserService(action.payload)
          .pipe(
            map(
              (user: User) =>
                new SaveNewUserActionSuccess(user)
            ),
            catchError((err) => of(new SaveNewUserActionError(err.message)))
          );
      })
    )
  );
}
