import { catchError, map } from 'rxjs/operators';
import { UserActionTypes } from './user.action.type';
import {
  UserActionUnion,
  DeleteUserActionSuccess,
  DeleteUserActionError,
} from './user.actions';
import { Observable, mergeMap, of } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { UserService } from './../../shared/services/user.service';
import { Injectable } from '@angular/core';

@Injectable()
export class UserDeleteEffect {
  constructor(private userService: UserService, private userActions: Actions) {}

  deleteUserEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.DELETE_USER),
      mergeMap((action: UserActionUnion) => {
        return this.userService.deleteUserService(action.payload.userID).pipe(
          map(() => new DeleteUserActionSuccess(action.payload,{})),
          catchError((err) => of(new DeleteUserActionError(err,{})))
        );
      })
    )
  );
}
