import { catchError, map } from 'rxjs/operators';
import { UserActionTypes } from './user.action.type';
import {
  UserActionUnion,
  GetAllUsersActionSuccess,
  GetAllUsersActionError,
} from './user.actions';
import { mergeMap, Observable, of } from 'rxjs';
import { UserService } from '../../shared/services/user.service';
import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { User } from 'src/app/shared/models/user/user.model';

@Injectable()
export class UserGetAllEffects {
  constructor(private userservice: UserService, private actions: Actions) {}

  userGetAllEffect: Observable<UserActionUnion> = createEffect(() =>
    this.actions.pipe(
      ofType(UserActionTypes.GET_ALL_USERS),
      mergeMap((action: UserActionUnion) => {
        return this.userservice.geAllUsersService().pipe(
          map((users: User[]) => new GetAllUsersActionSuccess(users, {})),
          catchError((err) => of(new GetAllUsersActionError(err, {})))
        );
      })
    )
  );
}
