import { Router } from '@angular/router';
import { MsgState } from './../../../../ngrx/msg.state';
import { CompanyType1 } from '../../../../shared/models/company/company.type1';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { CreateFormAddCompanyAction, SaveNewCompanyAction } from './../../../../ngrx/company/company.actions';
import { CompanyState } from './../../../../ngrx/company/company.state';
import { Component, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';
import { CompanyType2 } from 'src/app/shared/models/company/company.type2';

@Component({
  selector: 'app-company-create',
  templateUrl: './company-create.component.html',
  styleUrls: ['./company-create.component.css'],
})
export class CompanyCreateComponent implements OnInit {
  constructor(
    private store: Store<{ companyReducerSelector: CompanyState }>,
    private fb: FormBuilder,
    private router: Router
  ) {}

  companyState!: CompanyState;
  companyFormGroup!: FormGroup;
  companyTypes1 = CompanyType1;
  companyTypes2 = CompanyType2;
  msgState = MsgState;

  ngOnInit(): void {
    this.store.dispatch(new CreateFormAddCompanyAction({}));
    this.store.subscribe({
      next: (state) => {
        this.companyState = state.companyReducerSelector;

        this.companyFormGroup = this.fb.group({
          id: [0, Validators.required],
          companyName: [
            '',
            [
              Validators.required,
              Validators.minLength(4),
              Validators.maxLength(20),
            ],
          ],
          companyType1: ['', Validators.required],
          companyType2: ['', Validators.required],
          creationDate: [''],
        });
      },
      error: (err) => {},
    });
  }

  onCompanyCreate(){
    if(window.confirm(this.msgState.CONFIRM_ADD)){
      this.store.dispatch(new SaveNewCompanyAction(this.companyFormGroup.value))
    }
  }
  onSaveOk(){
    this.router.navigateByUrl("/companies-management")
  }
}
