import { catchError } from 'rxjs/operators';
import { MyServerErrorHandler } from './serverErrorHandler';
import { Address } from './../models/user/address.model';
import { Observable } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';

@Injectable({
  providedIn: 'root',
})
export class AddressService {
  constructor(
    private httpClient: HttpClient,
    private serverErrorHandler: MyServerErrorHandler
  ) {}


  getAllAddressesService(): Observable<Array<Address>> {
    let addresses: Observable<Array<Address>> =
    this.httpClient.get<Array<Address>>(environment.ADDRESSES_SERVER + '/addresses');
    return addresses.pipe(
      catchError(this.serverErrorHandler.handleError)
    );
  }

  createAddressService(address: Address): Observable<Address> {
    return this.httpClient.post<Address>(environment.ADDRESSES_SERVER + '/addresses', address)
    .pipe( catchError(this.serverErrorHandler.handleError));
  }

  getAddressByIDService(addressID: number): Observable<Address> {
    return this.httpClient.get<Address>(environment.ADDRESSES_SERVER + '/addresses/' + addressID
    ).pipe(
      catchError(this.serverErrorHandler.handleError)
    );
  }

  updateAddressService(address: Address): Observable<Address> {
    return this.httpClient.put<Address>(
      environment.ADDRESSES_SERVER + '/addresses/' + address.addressID,address).pipe(
      catchError(this.serverErrorHandler.handleError)
    );
  }

  deleteAddressService(addressID: number) {
    return this.httpClient.delete(environment.ADDRESSES_SERVER + '/addresses/' + addressID).pipe(
      catchError(this.serverErrorHandler.handleError)
    );
  }
}
